@@ -10,6 +10,7 @@
 #               2) Joachim Loehr private communication
 #               3) http://homepage3.nifty.com/kamisaka/makernote/makernote_casio.htm
 #               4) http://www.gvsoft.homedns.org/exif/makernote-casio.html
+#               5) Robert Chi private communication (EX-F1)
 #               JD) Jens Duttke private communication
 #------------------------------------------------------------------------------
 
@@ -19,7 +20,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.28';
+$VERSION = '1.31';
 
 # older Casio maker notes (ref 1)
 %Image::ExifTool::Casio::Main = (
@@ -484,6 +485,7 @@ $VERSION = '1.28';
             '2 0 0' => 'Mist Removal',
             '3 0 0' => 'Vivid Landscape',
             # have also seen '1 1 1', '2 2 4', '4 3 3', '4 4 4'
+            # '0 0 14' and '0 0 42' - premium auto night shot (EX-Z2300)
         },
     },
     0x3000 => {
@@ -536,6 +538,7 @@ $VERSION = '1.28';
            3 => 'Single-Area Auto Focus',
            5 => 'Infinity', #PH
            6 => 'Multi-Area Auto Focus',
+           8 => 'Super Macro', #PH (EX-Z2300)
         },
     },
     0x3006 => {
@@ -631,6 +634,9 @@ $VERSION = '1.28';
         PrintConv => {
             0 => 'Normal',
             8 => 'Silent Movie',
+            45 => 'Premium Auto', # (EX-2300)
+            47 => 'Painting', # (EX-2300)
+            49 => 'Crayon Drawing', # (EX-2300)
         },
     },
     0x301c => { #3
@@ -653,6 +659,7 @@ $VERSION = '1.28';
             # 3 observed in MOV videos (EX-V7)
             # (newer models write 2 numbers here - PH)
             '0 0' => 'Off', #PH
+            # have seen '0 1' for EX-Z2000 which has 5 modes: Auto, Camera AS, Image AS, DEMO, Off
             '16 0' => 'Slow Shutter', #PH (EX-Z77)
             '18 0' => 'Anti-Shake', #PH (EX-Z77)
             '20 0' => 'High Sensitivity', #PH (EX-Z77)
@@ -691,6 +698,36 @@ $VERSION = '1.28';
             1 => 'Makeup',
             2 => 'Mist Removal',
             3 => 'Vivid Landscape',
+            16 => 'Art Shot', # (EX-Z2300)
+        },
+    },
+    0x3103 => { #5
+        Name => 'DriveMode',
+        Writable => 'int16u',
+        PrintConvColumns => 2,
+        PrintConv => {
+            OTHER => sub {
+                # handle new values of future models
+                my ($val, $inv) = @_;
+                return $val =~ /(\d+)/ ? $1 : undef if $inv;
+                return "Continuous ($val fps)";
+            },
+            0 => 'Single Shot', #PH (NC)
+            1 => 'Continuous Shooting', # (1 fps for the EX-F1)
+            2 => 'Continuous (2 fps)',
+            3 => 'Continuous (3 fps)',
+            4 => 'Continuous (4 fps)',
+            5 => 'Continuous (5 fps)',
+            6 => 'Continuous (6 fps)',
+            7 => 'Continuous (7 fps)',
+            10 => 'Continuous (10 fps)',
+            12 => 'Continuous (12 fps)',
+            15 => 'Continuous (15 fps)',
+            20 => 'Continuous (20 fps)',
+            30 => 'Continuous (30 fps)',
+            40 => 'Continuous (40 fps)', #PH (EX-FH25)
+            60 => 'Continuous (60 fps)',
+            240 => 'Auto-N',
         },
     },
     0x4001 => { #PH (AVI videos)
@@ -717,6 +754,50 @@ $VERSION = '1.28';
     },
 );
 
+# Casio APP1 QVCI segment found in QV-7000SX images (ref PH)
+%Image::ExifTool::Casio::QVCI = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FIRST_ENTRY => 0,
+    NOTES => q{
+        This information is found in the APP1 QVCI segment of JPEG images from the
+        Casio QV-7000SX.
+    },
+    0x2c => {
+        Name => 'CasioQuality',
+        PrintConv => {
+            1 => 'Economy',
+            2 => 'Normal',
+            3 => 'Fine',
+            4 => 'Super Fine',
+        },
+    },
+    0x37 => {
+        Name => 'FocalRange',
+        Unknown => 1,
+    },
+    0x4d => {
+        Name => 'DateTimeOriginal',
+        Description => 'Date/Time Original',
+        Format => 'string[20]',
+        Groups => { 2 => 'Time' },
+        ValueConv => '$val=~tr/./:/; $val=~s/(\d+:\d+:\d+):/$1 /; $val',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    0x62 => {
+        Name => 'ModelType',
+        Format => 'string[7]',
+    },
+    0x72 => { # could be serial number or manufacture date in form YYMMDDxx ?
+        Name => 'ManufactureIndex',
+        Format => 'string[9]',
+    },
+    0x7c => {
+        Name => 'ManufactureCode',
+        Format => 'string[9]',
+    },
+);
+
 # tags in Casio AVI videos (ref PH)
 %Image::ExifTool::Casio::AVI = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -765,7 +846,7 @@ under the same terms as Perl itself.
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to Joachim Loehr for adding support for the type 2 maker notes, and
-Jens Duttke for decoding some tags.
+Jens Duttke and Robert Chi for decoding some tags.
 
 =head1 SEE ALSO
 
