@@ -14,7 +14,7 @@ package Image::ExifTool::IPTC;
 use strict;
 use vars qw($VERSION $AUTOLOAD %iptcCharset);
 
-$VERSION = '1.32';
+$VERSION = '1.38';
 
 %iptcCharset = (
     "\x1b%G"  => 'UTF8',
@@ -193,7 +193,7 @@ my %fileFormat = (
             affects the decoding of string values in the Application and NewsPhoto
             records.  This tag is marked as "unsafe" to prevent it from being copied by
             default in a group operation because existing tags in the destination image
-            may use a different encoding. When creating a new IPTC record from scratch,
+            may use a different encoding.  When creating a new IPTC record from scratch,
             it is suggested that this be set to "UTF8" if special characters are a
             possibility
         },
@@ -488,14 +488,14 @@ my %fileFormat = (
         Name => 'Caption-Abstract',
         Format => 'string[0,2000]',
     },
-    121 => { # (format not certain)
+    121 => {
         Name => 'LocalCaption',
-        Format => 'string[0,256]',
+        Format => 'string[0,256]', # (guess)
         Notes => q{
             I haven't found a reference for the format of tags 121, 184-188 and
             225-232, so I have just make them writable as strings with
             reasonable length.  Beware that if this is wrong, other utilities
-            won't be able to read these tags as written by ExifTool.
+            won't be able to read these tags as written by ExifTool
         },
     },
     122 => {
@@ -567,25 +567,25 @@ my %fileFormat = (
         Name => 'AudioOutcue',
         Format => 'string[0,64]',
     },
-    184 => { # (format not certain)
+    184 => {
         Name => 'JobID',
-        Format => 'string[0,64]',
+        Format => 'string[0,64]', # (guess)
     },
-    185 => { # (format not certain)
+    185 => {
         Name => 'MasterDocumentID',
-        Format => 'string[0,256]',
+        Format => 'string[0,256]', # (guess)
     },
-    186 => { # (format not certain)
+    186 => {
         Name => 'ShortDocumentID',
-        Format => 'string[0,64]',
+        Format => 'string[0,64]', # (guess)
     },
-    187 => { # (format not certain)
+    187 => {
         Name => 'UniqueDocumentID',
-        Format => 'string[0,128]',
+        Format => 'string[0,128]', # (guess)
     },
-    188 => { # (format not certain)
+    188 => {
         Name => 'OwnerID',
-        Format => 'string[0,128]',
+        Format => 'string[0,128]', # (guess)
     },
     200 => {
         Name => 'ObjectPreviewFileFormat',
@@ -620,25 +620,31 @@ my %fileFormat = (
             return $val;
         },
     },
-    225 => { # (format not certain)
+    225 => {
         Name => 'ClassifyState',
-        Format => 'string[0,64]',
+        Format => 'string[0,64]', # (guess)
     },
-    228 => { # (format not certain)
+    228 => {
         Name => 'SimilarityIndex',
-        Format => 'string[0,32]',
+        Format => 'string[0,32]', # (guess)
     },
-    230 => { # (format not certain)
+    230 => {
         Name => 'DocumentNotes',
-        Format => 'string[0,1024]',
+        Format => 'string[0,1024]', # (guess)
     },
-    231 => { # (format not certain)
+    231 => {
         Name => 'DocumentHistory',
-        Format => 'string[0,256]',
+        Format => 'string[0,256]', # (guess)
     },
-    232 => { # (format not certain)
+    232 => {
         Name => 'ExifCameraInfo',
-        Format => 'string[0,4096]',
+        Format => 'string[0,4096]', # (guess)
+    },
+    255 => { #PH
+        Name => 'CatalogSets',
+        List => 1,
+        Format => 'string[0,256]', # (guess)
+        Notes => 'written by iView MediaPro',
     },
 );
 
@@ -877,6 +883,34 @@ my %fileFormat = (
     },
 );
 
+# IPTC Composite tags
+%Image::ExifTool::IPTC::Composite = (
+    GROUPS => { 2 => 'Image' },
+    DateTimeCreated => {
+        Description => 'Date/Time Created',
+        Groups => { 2 => 'Time' },
+        Require => {
+            0 => 'IPTC:DateCreated',
+            1 => 'IPTC:TimeCreated',
+        },
+        ValueConv => '"$val[0] $val[1]"',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    DigitalCreationDateTime => {
+        Description => 'Digital Creation Date/Time',
+        Groups => { 2 => 'Time' },
+        Require => {
+            0 => 'IPTC:DigitalCreationDate',
+            1 => 'IPTC:DigitalCreationTime',
+        },
+        ValueConv => '"$val[0] $val[1]"',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+);
+
+# add our composite tags
+Image::ExifTool::AddCompositeTags('Image::ExifTool::IPTC');
+
 
 #------------------------------------------------------------------------------
 # AutoLoad our writer routines when necessary
@@ -902,27 +936,27 @@ sub PrintCodedCharset($)
 #------------------------------------------------------------------------------
 # Handle CodedCharacterSet
 # Inputs: 0) ExifTool ref, 1) CodedCharacterSet value
-# Returns: external character set if translation required (or 'bad' if unknown)
+# Returns: IPTC character set if translation required (or 'bad' if unknown)
 sub HandleCodedCharset($$)
 {
     my ($exifTool, $val) = @_;
-    my $xlat = $exifTool->Options('Charset');
-    if ($iptcCharset{$val}) {
-        # no need to translate if destination is the same
-        undef $xlat if $xlat eq $iptcCharset{$val};
-    } elsif ($val =~ /^\x1b\x25/) {
-        # some unknown character set invoked
-        $xlat = 'bad';  # flag unsupported coding
-    } else {
-        # translate all other codes as Latin
-        undef $xlat if $xlat eq 'Latin';
+    my $xlat = $iptcCharset{$val};
+    unless ($xlat) {
+        if ($val =~ /^\x1b\x25/) {
+            # some unknown character set invoked
+            $xlat = 'bad';  # flag unsupported coding
+        } else {
+            $xlat = $exifTool->Options('CharsetIPTC');
+        }
     }
+    # no need to translate if Charset is the same
+    undef $xlat if $xlat eq $exifTool->Options('Charset');
     return $xlat;
 }
 
 #------------------------------------------------------------------------------
 # Encode or decode coded string
-# Inputs: 0) ExifTool ref, 1) value ptr, 2) destination charset ('Latin','UTF8' or 'bad')
+# Inputs: 0) ExifTool ref, 1) value ptr, 2) IPTC charset (or 'bad') ref
 #         3) flag set to decode (read) value from IPTC
 # Updates value on return
 sub TranslateCodedString($$$$)
@@ -932,23 +966,27 @@ sub TranslateCodedString($$$$)
     if ($$xlatPtr eq 'bad') {
         $exifTool->Warn('Some IPTC characters not converted (unsupported CodedCharacterSet)');
         undef $$xlatPtr;
-    } elsif ($$xlatPtr eq 'Latin' xor $read) {
+    } elsif (not $read) {
+        $$valPtr = $exifTool->Decode($$valPtr, undef, undef, $$xlatPtr);
+    } elsif ($$valPtr !~ /[\x14\x15\x1b]/) {
+        $$valPtr = $exifTool->Decode($$valPtr, $$xlatPtr);
+    } elsif (not $$exifTool{WarnShift2022}) {
         # don't yet support reading ISO 2022 shifted character sets
-        if (not $read or $$valPtr !~ /[\x14\x15\x1b]/) {
-            # convert from Latin to UTF-8
-            my $val = Image::ExifTool::Latin2Unicode($$valPtr,'n');
-            $$valPtr = Image::ExifTool::Unicode2UTF8($val,'n');
-        } elsif (not $$exifTool{WarnShift2022}) {
-            $exifTool->Warn('Some IPTC characters not converted (ISO 2022 shifting not supported)');
-            $$exifTool{WarnShift2022} = 1;
-        }
-    } else {
-        # convert from UTF-8 to Latin
-        my $val = Image::ExifTool::UTF82Unicode($$valPtr,'n',$exifTool);
-        $$valPtr = Image::ExifTool::Unicode2Latin($val,'n',$exifTool);
+        $exifTool->Warn('Some IPTC characters not converted (ISO 2022 shifting not supported)');
+        $$exifTool{WarnShift2022} = 1;
     }
 }
 
+#------------------------------------------------------------------------------
+# Is this IPTC in a standard location?
+# Inputs: 0) Current metadata path string
+# Returns: true if path is standard
+sub IsStandardIPTC($)
+{
+    my $path = shift;
+    return $path =~ /^(JPEG-APP13-Photoshop-IPTC|TIFF-IFD0-IPTC|PSD-IPTC|MIE-IPTC)$/
+}
+
 #------------------------------------------------------------------------------
 # get IPTC info
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference
@@ -965,26 +1003,38 @@ sub ProcessIPTC($$$)
     my $success = 0;
     my ($lastRec, $recordPtr, $recordName);
 
-    # begin by assuming IPTC is Latin (so no translation if Charset is Latin)
-    my $xlat = $exifTool->Options('Charset');
-    undef $xlat if $xlat eq 'Latin';
-
     $verbose and $dirInfo and $exifTool->VerboseDir('IPTC', 0, $$dirInfo{DirLen});
+
     if ($tagTablePtr eq \%Image::ExifTool::IPTC::Main) {
+        # calculate MD5 if Digest::MD5 is available (for standard IPTC only)
+        my $path = $exifTool->MetadataPath();
+        if (IsStandardIPTC($path)) {
+            my $md5;
+            if (eval 'require Digest::MD5') {
+                if ($pos or $dirLen != length($$dataPt)) {
+                    $md5 = Digest::MD5::md5(substr $$dataPt, $pos, $dirLen);
+                } else {
+                    $md5 = Digest::MD5::md5($$dataPt);
+                }
+            } else {
+                # a zero digest indicates IPTC exists but we don't have Digest::MD5
+                $md5 = "\0" x 16;
+            }
+            $exifTool->FoundTag('CurrentIPTCDigest', $md5);
+        } elsif ($Image::ExifTool::MWG::strict and $$exifTool{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/) {
+            # ignore non-standard IPTC while in strict MWG compatibility mode
+            $exifTool->Warn("Ignored non-standard IPTC at $path");
+            return 1;
+        }
+        # set family 1 group name if multiple IPTC directories
         my $dirCount = ($exifTool->{DIR_COUNT}->{IPTC} || 0) + 1;
         $exifTool->{DIR_COUNT}->{IPTC} = $dirCount;
         $exifTool->{SET_GROUP1} = '+' . $dirCount if $dirCount > 1;
     }
-    # calculate MD5 if Digest::MD5 is available
-    if (eval 'require Digest::MD5') {
-        my $md5;
-        if ($pos or $dirLen != length($$dataPt)) {
-            $md5 = Digest::MD5::md5(substr $$dataPt, $pos, $dirLen);
-        } else {
-            $md5 = Digest::MD5::md5($$dataPt);
-        }
-        $exifTool->FoundTag('CurrentIPTCDigest', $md5);
-    }
+    # begin by assuming default IPTC encoding
+    my $xlat = $exifTool->Options('CharsetIPTC');
+    undef $xlat if $xlat eq $exifTool->Options('Charset');
+
     # quick check for improperly byte-swapped IPTC
     if ($dirLen >= 4 and substr($$dataPt, $pos, 1) ne "\x1c" and
                          substr($$dataPt, $pos + 3, 1) eq "\x1c")
@@ -1055,6 +1105,8 @@ sub ProcessIPTC($$$)
 
         my $tagInfo = $exifTool->GetTagInfo($recordPtr, $tag);
         my $format;
+        # (could use $$recordPtr{FORMAT} if no Format below, but don't do this to
+        #  be backward compatible with improperly written PhotoMechanic tags)
         $format = $$tagInfo{Format} if $tagInfo;
         # use logic to determine format if not specified
         unless ($format) {
@@ -1124,7 +1176,7 @@ image files.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
