@@ -14,9 +14,12 @@ use Image::ExifTool qw(:DataAccess);
 use Image::ExifTool::Exif;
 
 sub ProcessUnknown($$$);
+sub ProcessUnknownOrPreview($$$);
+sub ProcessCanon($$$);
+sub WriteUnknownOrPreview($$$);
 sub FixLeicaBase($$;$);
 
-$VERSION = '1.51';
+$VERSION = '1.61';
 
 my $debug;          # set to 1 to enabled debugging code
 
@@ -34,6 +37,7 @@ my $debug;          # set to 1 to enabled debugging code
         Condition => '$$self{Make} =~ /^Canon/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Canon::Main',
+            ProcessProc => \&ProcessCanon,
             ByteOrder => 'Unknown',
         },
     },
@@ -42,7 +46,7 @@ my $debug;          # set to 1 to enabled debugging code
         # do negative lookahead assertion just to get tags
         # in a nice order for documentation
         # (starts with an IFD)
-        Condition => '$$self{Make}=~/^CASIO(?! COMPUTER CO.,LTD)/',
+        Condition => '$$self{Make}=~/^CASIO/ and $$valPt!~/^(QVC|DCI)\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Casio::Main',
             ByteOrder => 'Unknown',
@@ -50,9 +54,9 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteCasio2',
-        # (starts with "QVC\0")
+        # (starts with "QVC\0" [Casio] or "DCI\0" [Concord])
         # (also found in AVI and MOV videos)
-        Condition => '$$self{Make}=~/^CASIO COMPUTER CO.,LTD/',
+        Condition => '$$valPt =~ /^(QVC|DCI)\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Casio::Type2',
             Start => '$valuePtr + 6',
@@ -181,7 +185,11 @@ my $debug;          # set to 1 to enabled debugging code
         # not much to key on here, but we know the
         # upper byte of the year should be 0x07:
         Name => 'MakerNoteKodak3',
-        Condition => '$$self{Make}=~/^EASTMAN KODAK/ and $$valPt=~/^.{12}\x07/s',
+        Condition => q{
+            $$self{Make} =~ /^EASTMAN KODAK/ and
+            $$valPt =~ /^(?!MM|II).{12}\x07/s and
+            $$valPt !~ /^(MM|II|AOC)/
+        },
         NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type3',
@@ -190,7 +198,11 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteKodak4',
-        Condition => '$$self{Make}=~/^Eastman Kodak/ and $$valPt=~/^.{41}JPG/s',
+        Condition => q{
+            $$self{Make} =~ /^Eastman Kodak/ and
+            $$valPt =~ /^.{41}JPG/s and
+            $$valPt !~ /^(MM|II|AOC)/
+        },
         NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type4',
@@ -283,10 +295,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKodak9',
         # test header and Kodak:DateTimeOriginal
-        Condition => q{
-            $$self{Make}=~/Kodak/i and
-            $$valPt =~ /^IIII.{16}\d{4}\/\d{2}\/\d{2} /s
-        },
+        Condition => '$$valPt =~ m{^IIII[\x02\x03]\0.{14}\d{4}/\d{2}/\d{2} }s',
         NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type9',
@@ -321,7 +330,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKyocera',
         # (starts with "KYOCERA")
-        Condition => '$$self{Make}=~/^KYOCERA/',
+        Condition => '$$valPt =~ /^KYOCERA/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Unknown::Main',
             Start => '$valuePtr + 22',
@@ -360,9 +369,8 @@ my $debug;          # set to 1 to enabled debugging code
         # /^\xd7/ - DiMAGE RD3000
         Name => 'MakerNoteMinolta3',
         Condition => '$$self{Make} =~ /^(Konica Minolta|Minolta)/i',
-        Notes => 'not EXIF-based',
         Binary => 1,
-        NotIFD => 1,
+        Notes => 'not EXIF-based',
     },
     {
         # this maker notes starts with a standard TIFF header at offset 0x0a
@@ -419,8 +427,8 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteLeica',
-        # (starts with "LEICA\0")
-        Condition => '$$self{Make} =~ /^LEICA/',
+        # (starts with "LEICA\0\0\0")
+        Condition => '$$self{Make} eq "LEICA"',
         SubDirectory => {
             # many Leica models use the same format as Panasonic
             TagTable => 'Image::ExifTool::Panasonic::Main',
@@ -430,8 +438,8 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteLeica2', # used by the M8
-        # (starts with "LEICA\0")
-        Condition => '$$self{Make} =~ /^Leica Camera AG/ and $$valPt =~ /^LEICA\0/',
+        # (starts with "LEICA\0\0\0")
+        Condition => '$$self{Make} =~ /^Leica Camera AG/ and $$valPt =~ /^LEICA\0\0\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Leica2',
             # (the offset base is different in JPEG and DNG images, but we
@@ -445,18 +453,58 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteLeica3', # used by the R8 and R9
-        # (starts with "LEICA\0")
-        Condition => '$$self{Make} =~ /^Leica Camera AG/',
+        # (starts with IFD)
+        Condition => '$$self{Make} =~ /^Leica Camera AG/ and $$valPt !~ /^LEICA/ and $$self{Model} ne "S2"',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Leica3',
             Start => '$valuePtr',
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNoteLeica4', # used by the M9
+        # (M9 starts with "LEICA0\x03\0")
+        Condition => '$$self{Make} =~ /^Leica Camera AG/ and $$valPt =~ /^LEICA0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Panasonic::Leica4',
+            Start => '$valuePtr + 8',
+            Base => '$start - 8', # (yay! Leica fixed the M8 problem)
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNoteLeica5', # used by the X1
+        # (X1 starts with "LEICA\0\x01\0", Make is "LEICA CAMERA AG")
+        Condition => '$$valPt =~ /^LEICA\0\x01\0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Panasonic::Leica5',
+            Start => '$valuePtr + 8',
+            Base => '$start - 8',
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNoteLeica6', # used by the S2 (CAUTION: this tag name is special cased in the code)
+        # (S2 starts with "LEICA\0\x02\xff", Make is "LEICA CAMERA AG",
+        #  but maker notes aren't loaded at the time this is tested)
+        Condition => '$$self{Model} eq "S2"',
+        DataTag => 'LeicaTrailer',  # (generates fixup name for this tag)
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Panasonic::Leica6',
+            Start => '$valuePtr + 8',
+            ByteOrder => 'Unknown',
+            # NOTE: Leica uses absolute file offsets when this maker note is stored
+            # as a JPEG trailer -- this case is handled by ProcessLeicaTrailer in
+            # Panasonic.pm, and any "Base" defined here is ignored for this case.
+            # ExifTool may also create S2 maker notes inside the APP1 segment when
+            # copying from other files, and for this the normal EXIF offsets are used,
+            # Base should not be defined!
+        },
+    },
     {
         Name => 'MakerNotePanasonic',
         # (starts with "Panasonic\0")
-        Condition => '$$self{Make} =~ /^Panasonic/ and $$valPt!~/^MKE/',
+        Condition => '$$valPt=~/^Panasonic/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Main',
             Start => '$valuePtr + 12',
@@ -466,7 +514,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNotePanasonic2',
         # (starts with "Panasonic\0")
-        Condition => '$$self{Make} =~ /^Panasonic/',
+        Condition => '$$self{Make}=~/^Panasonic/ and $$valPt=~/^MKE/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Type2',
             ByteOrder => 'LittleEndian',
@@ -547,16 +595,35 @@ my $debug;          # set to 1 to enabled debugging code
         },
     },
     {
-        # PreviewImage (Samsung Digimax plus some other types of cameras)
-        %Image::ExifTool::previewImageTagInfo,
-        Condition => '$$valPt =~ /^\xff\xd8\xff\xdb/ and $$self{Make}!~/(SONY|SANYO|SIGMA)/i',
-        Notes => 'Samsung preview image',
+        Name => 'MakerNoteSamsung1a',
+        # Samsung STMN maker notes WITHOUT PreviewImage
+        Condition => '$$valPt =~ /^STMN\d{3}.\0{4}/s',
+        Binary => 1,
+        Notes => 'Samsung "STMN" maker notes without PreviewImage',
     },
     {
-        Name => 'MakerNoteSamsung',
+        Name => 'MakerNoteSamsung1b',
+        # Samsung STMN maker notes WITH PreviewImage
         Condition => '$$valPt =~ /^STMN\d{3}/',
-        Binary => 1,
-        Notes => 'Samsung unknown maker notes',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Samsung::Type1',
+        },
+    },
+    {
+        Name => 'MakerNoteSamsung2',
+        # Samsung EXIF-format maker notes
+        Condition => q{
+            $$self{Make} eq 'SAMSUNG' and ($$self{TIFF_TYPE} eq 'SRW' or
+            $$valPt=~/^(\0.\0\x01\0\x07\0{3}\x04|.\0\x01\0\x07\0\x04\0{3})0100/s)
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Samsung::Type2',
+            # Samsung is very inconsistent here, and uses absolute offsets for some
+            # models and relative offsets for others, so process as Unknown
+            ProcessProc => \&ProcessUnknown,
+            FixBase => 1,
+            ByteOrder => 'Unknown',
+        },
     },
     {
         Name => 'MakerNoteSanyo',
@@ -643,6 +710,16 @@ my $debug;          # set to 1 to enabled debugging code
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNoteSonyEricsson',
+        Condition => '$$valPt =~ /^SEMC MS\0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Sony::Ericsson',
+            Start => '$valuePtr + 20',
+            Base => '$start - 8',
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         Name => 'MakerNoteSonySRF',
         Condition => '$$self{Make}=~/^SONY/',
@@ -652,13 +729,24 @@ my $debug;          # set to 1 to enabled debugging code
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNoteUnknownText',
+        Condition => '$$valPt =~ /^[\x09\x0d\x0a\x20-\x7e]+\0*$/',
+        Notes => 'unknown text-based maker notes',
+        # show as binary if it is too long
+        ValueConv => 'length($val) > 64 ? \$val : $val',
+        ValueConvInv => '$val',
+    },
     {
         Name => 'MakerNoteUnknown',
+        PossiblePreview => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Unknown::Main',
-            ProcessProc => \&ProcessUnknown,
+            ProcessProc => \&ProcessUnknownOrPreview,
+            WriteProc => \&WriteUnknownOrPreview,
             ByteOrder => 'Unknown',
-        },
+            FixBase => 2,
+       },
     },
 );
 
@@ -669,7 +757,7 @@ foreach $tagInfo (@Image::ExifTool::MakerNotes::Main) {
     $$tagInfo{WriteGroup} = 'ExifIFD';
     $$tagInfo{Groups} = { 1 => 'MakerNotes' };
     next unless $$tagInfo{SubDirectory};
-    # set up this tag so we can write it
+    # make all SubDirectory tags block-writable
     $$tagInfo{Binary} = 1,
     $$tagInfo{MakerNotes} = 1;
 }
@@ -689,7 +777,9 @@ sub GetMakerNoteOffset($)
     my $model = $exifTool->{Model};
     my ($relative, @offsets);
 
-    # normally value data starts 4 bytes after end of directory, so this is the default
+    # normally value data starts 4 bytes after end of directory, so this is the default.
+    # offsets of 0 and 4 are always allowed even if not specified,
+    # but the first offset specified is the one used when writing
     if ($make =~ /^Canon/) {
         push @offsets, ($model =~ /\b(20D|350D|REBEL XT|Kiss Digital N)\b/) ? 6 : 4;
         # some Canon models (FV-M30, Optura50, Optura60) leave 24 unused bytes
@@ -704,7 +794,14 @@ sub GetMakerNoteOffset($)
     } elsif ($make =~ /^KYOCERA/) {
         push @offsets, 12;
     } elsif ($make =~ /^Leica Camera AG/) {
-        push @offsets, 6; # R8, R9 and M8
+        if ($model eq 'S2') {
+            # lots of empty space before first value in S2 images
+            push @offsets, 4, ($$exifTool{FILE_TYPE} eq 'JPEG' ? 286 : 274);
+        } elsif ($model =~ /^(R8|R9|M8)\b/) {
+            push @offsets, 6;
+        } else {
+            push @offsets, 4;
+        }
     } elsif ($make =~ /^OLYMPUS/ and $model =~ /^E-(1|300|330)\b/) {
         push @offsets, 16;
     } elsif ($make =~ /^OLYMPUS/ and
@@ -747,14 +844,15 @@ sub GetMakerNoteOffset($)
 
 #------------------------------------------------------------------------------
 # Get hash of value offsets / block sizes
-# Inputs: 0) Data pointer, 1) offset to start of directory
+# Inputs: 0) Data pointer, 1) offset to start of directory,
+#         2) hash ref to return value pointers based on tag ID
 # Returns: 0) hash reference: keys are offsets, values are block sizes
 #          1) same thing, but with keys adjusted for value-based offsets
 # Notes: Directory size should be validated before calling this routine
 # - calculates MIN and MAX offsets in entry-based hash
-sub GetValueBlocks($$)
+sub GetValueBlocks($$;$)
 {
-    my ($dataPt, $dirStart) = @_;
+    my ($dataPt, $dirStart, $tagPtr) = @_;
     my $numEntries = Get16u($dataPt, $dirStart);
     my ($index, $valPtr, %valBlock, %valBlkAdj, $end);
     for ($index=0; $index<$numEntries; ++$index) {
@@ -765,6 +863,7 @@ sub GetValueBlocks($$)
         my $size = $count * $Image::ExifTool::Exif::formatSize[$format];
         next if $size <= 4;
         $valPtr = Get32u($dataPt, $entry+8);
+        $tagPtr and $$tagPtr{Get16u($dataPt, $entry)} = $valPtr;
         # save location and size of longest block at this offset
         unless (defined $valBlock{$valPtr} and $valBlock{$valPtr} > $size) {
             $valBlock{$valPtr} = $size;
@@ -806,10 +905,10 @@ sub FixBase($$)
     my $dirName = $$dirInfo{DirName};
     my $fixBase = $exifTool->Options('FixBase');
     my $setBase = (defined $fixBase and $fixBase ne '') ? 1 : 0;
-    my ($fix, $fixedBy);
+    my ($fix, $fixedBy, %tagPtr);
 
     # get hash of value block positions
-    my ($valBlock, $valBlkAdj) = GetValueBlocks($dataPt, $dirStart);
+    my ($valBlock, $valBlkAdj) = GetValueBlocks($dataPt, $dirStart, \%tagPtr);
     return 0 unless %$valBlock;
     # get sorted list of value offsets
     my @valPtrs = sort { $a <=> $b } keys %$valBlock;
@@ -817,21 +916,13 @@ sub FixBase($$)
 # handle special case of Canon maker notes with TIFF footer containing original offset
 #
     if ($$exifTool{Make} =~ /^Canon/ and $$dirInfo{DirLen} > 8) {
-        my $trailerPos = $dirStart + $$dirInfo{DirLen} - 8;
-        my $footer = substr($$dataPt, $trailerPos, 8);
+        my $footerPos = $dirStart + $$dirInfo{DirLen} - 8;
+        my $footer = substr($$dataPt, $footerPos, 8);
         if ($footer =~ /^(II\x2a\0|MM\0\x2a)/ and  # check for TIFF footer
             substr($footer,0,2) eq GetByteOrder()) # validate byte ordering
         {
             my $oldOffset = Get32u(\$footer, 4);
             my $newOffset = $dirStart + $dataPos;
-            if ($$exifTool{HTML_DUMP}) {
-                my $filePos = ($$dirInfo{Base} || 0) + $dataPos + $trailerPos;
-                my $str = sprintf('Original maker note offset: 0x%.4x', $oldOffset);
-                if ($oldOffset != $newOffset) {
-                    $str .= sprintf("\nCurrent maker note offset: 0x%.4x", $newOffset);
-                }
-                $exifTool->HtmlDump($filePos, 8, '[Canon MakerNotes footer]', $str);
-            }
             if ($setBase) {
                 $fix = $fixBase;
             } else {
@@ -859,7 +950,7 @@ sub FixBase($$)
     }
 #
 # analyze value offsets to see if they need fixing.  The first task is to determine
-# the minimum valid offset used (this is tricky, because we have  to weed out bad
+# the minimum valid offset used (this is tricky, because we have to weed out bad
 # offsets written by some cameras)
 #
     my $minPt = $$dirInfo{MinOffset} = $valPtrs[0]; # if life were simple, this would be it
@@ -919,6 +1010,16 @@ sub FixBase($$)
             undef $entryBased;
             undef $relative;
         }
+        # use PrintIM tag to do special check for correct absolute offsets
+        if ($tagPtr{0xe00}) {
+            my $ptr = $tagPtr{0xe00} - $dataPos;
+            return 0 if $ptr > 0 and $ptr <= length($$dataPt) - 8 and
+                        substr($$dataPt, $ptr, 8) eq "PrintIM\0";
+        }
+        # allow a range of reasonable differences for Unknown maker notes
+        if ($$dirInfo{FixBase} and $$dirInfo{FixBase} == 2) {
+            return 0 if $diff >=0 and $diff <= 24;
+        }
         # (used for testing to extract differences)
         # $exifTool->FoundTag('Diff', $diff);
     }
@@ -1023,7 +1124,10 @@ sub LocateIFD($$)
 #
     if ($tagInfo and $$tagInfo{SubDirectory}) {
         my $subdir = $$tagInfo{SubDirectory};
-        unless ($$subdir{ProcessProc} and $$subdir{ProcessProc} eq \&ProcessUnknown) {
+        unless ($$subdir{ProcessProc} and 
+               ($$subdir{ProcessProc} eq \&ProcessUnknown or
+                $$subdir{ProcessProc} eq \&ProcessUnknownOrPreview))
+        {
             # look for the IFD at the "Start" specified in our SubDirectory information
             my $valuePtr = $dirStart;
             my $newStart = $dirStart;
@@ -1034,7 +1138,8 @@ sub LocateIFD($$)
             if ($$subdir{Base}) {
                 # calculate subdirectory start relative to $base for eval
                 my $start = $newStart + $$dirInfo{DataPos};
-                #### eval Base ($start)
+                my $base = $$dirInfo{Base} || 0;
+                #### eval Base ($start,$base)
                 my $baseShift = eval($$subdir{Base});
                 # shift directory base (note: we may do this again below
                 # if an OffsetPt is defined, but that doesn't matter since
@@ -1176,6 +1281,87 @@ sub FixLeicaBase($$;$)
     return $success;
 }
 
+#------------------------------------------------------------------------------
+# Process Canon maker notes
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub ProcessCanon($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    # identify Canon MakerNote footer in HtmlDump
+    # (this code moved from FixBase so it also works for Adobe MakN in DNG images)
+    if ($$exifTool{HTML_DUMP} and $$dirInfo{DirLen} > 8) {
+        my $dataPos = $$dirInfo{DataPos};
+        my $dirStart = $$dirInfo{DirStart} || 0;
+        my $footerPos = $dirStart + $$dirInfo{DirLen} - 8;
+        my $footer = substr(${$$dirInfo{DataPt}}, $footerPos, 8);
+        my $oldOffset = Get32u(\$footer, 4);
+        my $newOffset = $dirStart + $dataPos;
+        my $str = sprintf('Original maker note offset: 0x%.4x', $oldOffset);
+        if ($oldOffset != $newOffset) {
+            $str .= sprintf("\nCurrent maker note offset: 0x%.4x", $newOffset);
+        }
+        my $filePos = ($$dirInfo{Base} || 0) + $dataPos + $footerPos;
+        $exifTool->HDump($filePos, 8, '[Canon MakerNotes footer]', $str);
+    }
+    # process as normal
+    return Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+}
+
+#------------------------------------------------------------------------------
+# Process unknown maker notes or PreviewImage
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success, and updates $dirInfo if necessary for new directory
+sub ProcessUnknownOrPreview($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dirStart = $$dirInfo{DirStart};
+    my $dirLen = $$dirInfo{DirLen};
+    # check to see if this is a preview image
+    if ($dirLen > 6 and substr($$dataPt, $dirStart, 3) eq "\xff\xd8\xff") {
+        $exifTool->VerboseDir('PreviewImage');
+        if ($$exifTool{HTML_DUMP}) {
+            my $pos = $$dirInfo{DataPos} + $$dirInfo{Base} + $dirStart;
+            $exifTool->HDump($pos, $dirLen, '(MakerNotes:PreviewImage data)', "Size: $dirLen bytes")
+        }
+        $exifTool->FoundTag('PreviewImage', substr($$dataPt, $dirStart, $dirLen));
+        return 1;
+    }
+    return ProcessUnknown($exifTool, $dirInfo, $tagTablePtr);
+}
+
+#------------------------------------------------------------------------------
+# Write unknown maker notes or PreviewImage
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: directory data, '' to delete, or undef on error
+sub WriteUnknownOrPreview($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dirStart = $$dirInfo{DirStart};
+    my $dirLen = $$dirInfo{DirLen};
+    my $newVal;
+    # check to see if this is a preview image
+    if ($dirLen > 6 and substr($$dataPt, $dirStart, 3) eq "\xff\xd8\xff") {
+        if ($$exifTool{NEW_VALUE}{$Image::ExifTool::Extra{PreviewImage}}) {
+            # write or delete new preview (if deleted, it can't currently be added back again)
+            $newVal = $exifTool->GetNewValues('PreviewImage') || '';
+            if ($exifTool->Options('Verbose') > 1) {
+                $exifTool->VerboseValue("- MakerNotes:PreviewImage", substr($$dataPt, $dirStart, $dirLen));
+                $exifTool->VerboseValue("+ MakerNotes:PreviewImage", $newVal) if $newVal;
+            }
+            ++$$exifTool{CHANGED};
+        } else {
+            $newVal = substr($$dataPt, $dirStart, $dirLen);
+        }
+    } else {
+        # rewrite MakerNote IFD
+        $newVal = Image::ExifTool::Exif::WriteExif($exifTool, $dirInfo, $tagTablePtr);
+    }
+    return $newVal;
+}
+
 #------------------------------------------------------------------------------
 # Process unknown maker notes assuming it is in EXIF IFD format
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
@@ -1223,7 +1409,7 @@ maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
