@@ -97,8 +97,7 @@ sub PrintInvCodedCharset($)
 
 #------------------------------------------------------------------------------
 # validate raw values for writing
-# Inputs: 0) ExifTool object reference, 1) tagInfo hash reference,
-#         2) raw value reference
+# Inputs: 0) ExifTool object ref, 1) tagInfo hash ref, 2) raw value ref
 # Returns: error string or undef (and possibly changes value) on success
 sub CheckIPTC($$$)
 {
@@ -128,10 +127,13 @@ sub CheckIPTC($$$)
                 $len = $minlen;
             }
         }
-        if ($minlen) {
+        if (defined $minlen) {
             $maxlen or $maxlen = $minlen;
             return "String too short (minlen is $minlen)" if $len < $minlen;
-            return "String too long (maxlen is $maxlen)" if $len > $maxlen;
+            if ($len > $maxlen and not $exifTool->Options('IgnoreMinorErrors')) {
+                $$exifTool{CHECK_WARN} = "[minor] IPTC:$$tagInfo{Name} exceeds length limit (truncated)";
+                $$valPtr = substr($$valPtr, 0, $maxlen);
+            }
         }
     } else {
         return "Bad IPTC Format ($format)";
@@ -148,8 +150,9 @@ sub CheckIPTC($$$)
 sub FormatIPTC($$$$$;$)
 {
     my ($exifTool, $tagInfo, $valPtr, $xlatPtr, $rec, $read) = @_;
-    return unless $$tagInfo{Format};
-    if ($$tagInfo{Format} =~ /^int(\d+)/) {
+    my $format = $$tagInfo{Format} || $$tagInfo{Table}{FORMAT};
+    return unless $format;
+    if ($format =~ /^int(\d+)/) {
         if ($read) {
             my $len = length($$valPtr);
             if ($len <= 8) {    # limit integer conversion to 8 bytes long
@@ -170,7 +173,7 @@ sub FormatIPTC($$$$$;$)
                 $$valPtr = pack('N', $$valPtr);
             }
         }
-    } elsif ($$tagInfo{Format} =~ /^string/) {
+    } elsif ($format =~ /^string/) {
         if ($rec == 1) {
             if ($$tagInfo{Name} eq 'CodedCharacterSet') {
                 $$xlatPtr = HandleCodedCharset($exifTool, $$valPtr);
@@ -188,7 +191,7 @@ sub FormatIPTC($$$$$;$)
 sub IptcDate($)
 {
     my $val = shift;
-    unless ($val =~ s/.*(\d{4}):?(\d{2}):?(\d{2}).*/$1$2$3/) {
+    unless ($val =~ s/.*(\d{4}):?(\d{2}):?(\d{2}).*/$1$2$3/s) {
         warn "Invalid date format (use YYYY:MM:DD)\n";
         undef $val;
     }
@@ -202,21 +205,32 @@ sub IptcDate($)
 sub IptcTime($)
 {
     my $val = shift;
-    if ($val =~ /\s*\b(\d{1,2})(:?)(\d{2})(:?)(\d{2})(\S*)\s*$/ and ($2 or not $4)) {
-        $val = sprintf("%.2d%.2d%.2d",$1,$3,$5);
-        my $tz = $6;
+    if ($val =~ /(.*?)\b(\d{1,2})(:?)(\d{2})(:?)(\d{2})(\S*)\s*$/s and ($3 or not $5)) {
+        $val = sprintf("%.2d%.2d%.2d",$2,$4,$6);
+        my ($date, $tz) = ($1, $7);
         if ($tz =~ /([+-]\d{1,2}):?(\d{2})/) {
             $tz = sprintf("%+.2d%.2d",$1,$2);
-        } elsif ($tz !~ /Z/i) {
-            # use local system timezone by default (note: it is difficult to use
-            # the proper local timezone for this date/time because the date tag
-            # is written separately so we don't know what the local timezone offset
-            # really should be for this date/time)
-            my $now = time;
-            my @tm = localtime($now);
-            ($tz = Image::ExifTool::TimeZoneString(\@tm, $now)) =~ tr/://d;
+        } elsif ($tz =~ /Z/i) {
+            $tz = '+0000';  # UTC
         } else {
-            $tz = '+0000';  # don't know the time zone
+            # use local system timezone by default 
+            my (@tm, $time);
+            if ($date and $date =~ /^(\d{4}):(\d{2}):(\d{2})\s*$/ and eval 'require Time::Local') {
+                # we were given a date too, so determine the local timezone
+                # offset at the specified date/time
+                my @d = ($3,$2-1,$1-1900);
+                $val =~ /(\d{2})(\d{2})(\d{2})/;
+                @tm = ($3,$2,$1,@d);
+                $time = Image::ExifTool::TimeLocal(@tm);
+            } else {
+                # it is difficult to get the proper local timezone offset for this
+                # time because the date tag is written separately.  (The offset may be
+                # different on a different date due to daylight savings time.)  In this
+                # case the best we can do easily is to use the current timezone offset.
+                $time = time;
+                @tm = localtime($time);
+            }
+            ($tz = Image::ExifTool::TimeZoneString(\@tm, $time)) =~ tr/://d;
         }
         $val .= $tz;
     } else {
@@ -285,7 +299,10 @@ sub DoWriteIPTC($$$)
     # - improves speed
     # - avoids changing current MD5 digest unnecessarily
     # - avoids adding mandatory tags unless some other IPTC is changed
-    unless (exists $exifTool->{EDIT_DIRS}->{$$dirInfo{DirName}}) {
+    unless (exists $$exifTool{EDIT_DIRS}{$$dirInfo{DirName}} or
+        # standard IPTC tags in other locations should be edited too (ie. AFCP_IPTC)
+        ($tagTablePtr = \%Image::ExifTool::IPTC::Main and exists $$exifTool{EDIT_DIRS}{IPTC}))
+    {
         print $out "$$exifTool{INDENT}  [nothing changed]\n" if $verbose;
         return undef;
     }
@@ -298,9 +315,9 @@ sub DoWriteIPTC($$$)
     my $dirLen = $$dirInfo{DirLen};
     my ($tagInfo, %iptcInfo, $tag);
 
-    # begin by assuming IPTC is coded in Latin unless otherwise specified
-    my $xlat = $exifTool->Options('Charset');
-    undef $xlat if $xlat eq 'Latin';
+    # start by assuming default IPTC encoding
+    my $xlat = $exifTool->Options('CharsetIPTC');
+    undef $xlat if $xlat eq $exifTool->Options('Charset');
 
     # make sure our dataLen is defined (note: allow zero length directory)
     unless (defined $dirLen) {
@@ -447,7 +464,8 @@ sub DoWriteIPTC($$$)
                                 # generate our new entry
                                 my $entry = pack("CCCn", 0x1c, $lastRec, $mandTag, length($value));
                                 $newData .= $entry . $value;    # add entry to new IPTC data
-                                ++$exifTool->{CHANGED};
+                                # (don't mark as changed if just mandatory tags changed)
+                                # ++$exifTool->{CHANGED};
                             }
                         }
                     }
@@ -569,20 +587,52 @@ sub WriteIPTC($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     $exifTool or return 1;    # allow dummy access to autoload this package
+
     my $newData = DoWriteIPTC($exifTool, $dirInfo, $tagTablePtr);
-    return $newData unless eval 'require Digest::MD5';
-    my $dataPt;
-    if (defined $newData) {
-        $dataPt = \$newData;
-    } else {
-        $dataPt = $$dirInfo{DataPt};
-        if ($$dirInfo{DirStart} or length($$dataPt) != $$dirInfo{DirLen}) {
-            my $buff = substr($$dataPt, $$dirInfo{DirStart}, $$dirInfo{DirLen});
-            $dataPt = \$buff;
+
+    # calculate standard IPTC digests only if we are writing or deleting
+    # Photoshop:IPTCDigest with a value of 'new' or 'old'
+    while ($Image::ExifTool::Photoshop::iptcDigestInfo) {
+        my $nvHash = $exifTool->{NEW_VALUE}{$Image::ExifTool::Photoshop::iptcDigestInfo};
+        last unless defined $nvHash;
+        last unless IsStandardIPTC($exifTool->MetadataPath());
+        my @values = Image::ExifTool::GetNewValues($nvHash);
+        push @values, @{$$nvHash{DelValue}} if $$nvHash{DelValue};
+        my $new = grep /^new$/, @values;
+        my $old = grep /^old$/, @values;
+        last unless $new or $old;
+        unless (eval 'require Digest::MD5') {
+            $exifTool->Warn('Digest::MD5 must be installed to calculate IPTC digest');
+            last;
         }
+        my $dataPt;
+        if ($new) {
+            if (defined $newData) {
+                $dataPt = \$newData;
+            } else {
+                $dataPt = $$dirInfo{DataPt};
+                if ($$dirInfo{DirStart} or length($$dataPt) != $$dirInfo{DirLen}) {
+                    my $buff = substr($$dataPt, $$dirInfo{DirStart}, $$dirInfo{DirLen});
+                    $dataPt = \$buff;
+                }
+            }
+            # set NewIPTCDigest data member unless IPTC is being deleted
+            $$exifTool{NewIPTCDigest} = Digest::MD5::md5($$dataPt) if length $$dataPt;
+        }
+        if ($old) {
+            if ($new and not defined $newData) {
+                $$exifTool{OldIPTCDigest} = $$exifTool{NewIPTCDigest};
+            } elsif ($$dirInfo{DataPt}) { #(may be undef if creating new IPTC)
+                $dataPt = $$dirInfo{DataPt};
+                if ($$dirInfo{DirStart} or length($$dataPt) != $$dirInfo{DirLen}) {
+                    my $buff = substr($$dataPt, $$dirInfo{DirStart}, $$dirInfo{DirLen});
+                    $dataPt = \$buff;
+                }
+                $$exifTool{OldIPTCDigest} = Digest::MD5::md5($$dataPt) if length $$dataPt;
+            }
+        }
+        last;
     }
-    # set NewIPTCDigest data member unless IPTC is being deleted
-    $$exifTool{NewIPTCDigest} = Digest::MD5::md5($$dataPt) if length $$dataPt;
     return $newData;
 }
 
@@ -606,7 +656,7 @@ seldom-used routines.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
